#' @templateVar old GatingSet2flowJo
#' @templateVar new gatingset_to_flowjo
#' @template template-depr_pkg
NULL

#' @export
GatingSet2flowJo <- function(...){
  .Deprecated("gatingset_to_flowjo")
  gatingset_to_flowjo(...)
}

#' Convert a GatingSet to flowJo workspace 
#' 
#' It is a R wrapper for the docker app
#' (https://hub.docker.com/r/wjiang2/gs-to-flowjo)
#'
#' @name gatingset_to_flowjo
#' @aliases GatingSet2flowJo
#' @param gs a GatingSet object or a folder contains the GatingSet archive (generated by previous \code{\link{save_gs}} call)
#' @param outFile the workspace file path to write
#' @param docker_img the docker image that does the actual work
#' @param ... other arguments passed to \code{\link{save_gs}}
#' @return nothing
#' @examples
#' \dontrun{
#' library(flowWorkspace)
#'
#' path <- system.file("extdata",package="flowWorkspaceData")
#' gs_path <- list.files(path, pattern = "gs_manual",full = TRUE)
#' gs <- load_gs(gs_path)
#'
#' #output to flowJo
#' outFile <- tempfile(fileext = ".wsp")
#' gatingset_to_flowjo(gs, outFile)
#' 
#' #or directly use the archive as the input (to avoid the extra copying inside of the wrapper)
#' gatingset_to_flowjo(gs_path, outFile)
#' }
#' @importFrom flowWorkspace gs_clone gs_update_channels pData<- cs_unlock cs_lock gs_copy_tree_only cs_load_meta 
#' @export
#' @rdname gatingset_to_flowjo
gatingset_to_flowjo <- function(gs, outFile, showHidden = FALSE, docker_img = "wjiang2/gs-to-flowjo", ...){
  res <- check_docker_status(docker_img)
  if(res!="ok")
    stop(res)
  v1 <- packageVersion("cytolib")
  v2 <- system2("docker", paste0("run ", docker_img, " --cytolib-version"), stdout = TRUE)
  if(v1!=v2)
    warning("docker image '", docker_img, "' is built with different cytolib version of from R package: ", v2, " vs ", v1)
  
  if(is(gs, "GatingSet"))
  {
    tmp <- tempfile()
    suppressMessages(save_gs(gs, tmp, ...))#todo:fix link="cdf"
    
  }else
    tmp <- gs
  
  res <- suppressWarnings(system2("docker"
                                  , paste0("run"
                                           , " -v ", tmp, ":/gs"
                                           , " -v ", normalizePath(dirname(outFile)), ":/out "
                                           , docker_img
                                           , " --src=/gs --dest=/out/", basename(outFile)
                                           , " --showHidden=", showHidden)
                                  , stderr = TRUE)
                          )
  if(length(res) > 0)
    stop(res)
}

check_docker_status <- function(docker_img = "wjiang2/gs-to-flowjo"){
  errcode <- system2("command", " -v docker", stdout = FALSE)
  if(errcode!=0)
    return("'docker' command is not found! ")
  
  errcode <- system2("docker", " info", stdout = FALSE, stderr = FALSE)
  if(errcode!=0)
    return("'docker' is not running properly! ")
  
  
  errcode <- system2("docker", paste0("  image inspect ", docker_img), stdout = FALSE, stderr = FALSE)
  if(errcode!=0)
    return(paste0("docker image '", docker_img, "' is present! "))
  
  return("ok")
}
